package com.jinhaiqimeng.sysManageMicroservice.web.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jinhaiqimeng.sysManageMicroservice.result.ResponseResult;
import com.jinhaiqimeng.sysManageMicroservice.util.DateUtils;
import com.jinhaiqimeng.sysManageMicroservice.web.dto.IdDTO;
import com.jinhaiqimeng.sysManageMicroservice.web.dto.PushMessageListDTO;
import com.jinhaiqimeng.sysManageMicroservice.web.dto.PushMessageSaveDTO;
import com.jinhaiqimeng.sysManageMicroservice.web.dto.PushMessageUpdateDTO;
import com.jinhaiqimeng.sysManageMicroservice.web.entity.PushMessage;
import com.jinhaiqimeng.sysManageMicroservice.web.dao.PushMessageDao;
import com.jinhaiqimeng.sysManageMicroservice.web.service.PushMessageService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jinhaiqimeng.sysManageMicroservice.web.vo.PushMessageVO;
import com.jinhaiqimeng.sysManageMicroservice.web.vo.common.ListVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 消息推送表 服务实现类
 * </p>
 *
 * @author zp
 * @since 2023-12-07
 */
@Service
public class PushMessageServiceImpl extends ServiceImpl<PushMessageDao, PushMessage> implements PushMessageService {

    @Override
    public ResponseResult<ListVO<PushMessage>> pushMessageList(PushMessageListDTO dto) {
        QueryWrapper<PushMessage> pushMessageVOQueryWrapper = new QueryWrapper<>();
        pushMessageVOQueryWrapper.lambda()
                .eq(dto.getMessagePushStatus()==1||dto.getMessagePushStatus()==2,PushMessage::getMessagePushStatus,dto.getMessagePushStatus())
                .between(StrUtil.isNotBlank(dto.getMessageOperationStartTime())&&StrUtil.isNotBlank(dto.getMessageOperationEndTime()),
                        PushMessage::getMessageOperationTime,dto.getMessageOperationStartTime(),dto.getMessageOperationEndTime())
                .between(StrUtil.isNotBlank(dto.getMessagePushStartTime())&&StrUtil.isNotBlank(dto.getMessagePushEndTime()),
                        PushMessage::getMessagePushTime,dto.getMessagePushStartTime(),dto.getMessagePushEndTime())
                .like(StrUtil.isNotBlank(dto.getMessageTitle()),PushMessage::getMessageTitle,dto.getMessageTitle())
                .orderByDesc(PushMessage::getMessageOperationTime);


        Page<PushMessage> reportPage = new Page<PushMessage>(dto.getPageNum(), dto.getPageSize());
        Page<PushMessage> page = page(reportPage,pushMessageVOQueryWrapper);
        List<PushMessage> records = page.getRecords();
        return ResponseResult.success(new ListVO<>(records,page.getTotal(),(int)page.getCurrent()));
    }

    @Override
    public ResponseResult savePushMessage(PushMessageSaveDTO dto) {
        PushMessage pushMessage=new PushMessage();
        pushMessage.setMessagePushStatus(2);
        pushMessage.setMessagePushTime(DateUtils.getLocalDateTime(dto.getMessagePushTime()));
        pushMessage.setMessageStatus(1);
        pushMessage.setMessageTitle(dto.getMessageTitle());
        pushMessage.setMessageDescription(dto.getMessageDescription());
        pushMessage.setMessageOperationTime(LocalDateTime.now());
        save(pushMessage);
        return ResponseResult.success();
    }

    @Override
    public ResponseResult<PushMessageVO> getPushMessage(IdDTO dto) {
        PushMessage pushMessage=getById(dto.getId());
        PushMessageVO pushMessageVO=new PushMessageVO();
        BeanUtils.copyProperties(pushMessage,pushMessageVO);
        return ResponseResult.success(pushMessageVO);
    }

    @Override
    public ResponseResult updatePushMessage(PushMessageUpdateDTO dto) {
        PushMessage pushMessage = getById(dto.getMessageld());
        if (pushMessage==null){
            return ResponseResult.failed("未找到消息推送信息!");
        }
        LocalDateTime localDateTime=null;
        if(StringUtils.isNotBlank(dto.getMessagePushTime()))
            localDateTime=DateUtils.getLocalDateTime(dto.getMessagePushTime());

        UpdateWrapper<PushMessage> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda()
                .set(StringUtils.isNotBlank(dto.getMessageTitle()),PushMessage::getMessageTitle,dto.getMessageTitle())
                .set(StringUtils.isNotBlank(dto.getMessageDescription()),PushMessage::getMessageDescription,dto.getMessageDescription())
                .set(StringUtils.isNotBlank(dto.getMessagePushTime()),PushMessage::getMessagePushTime,localDateTime)
                .eq(PushMessage::getId,dto.getMessageld());
        update(updateWrapper);
        return ResponseResult.success();
    }

    @Override
    public ResponseResult enable(IdDTO dto) {
        ResponseResult<Object> failed = updateMessageStatus(dto,1);
        if (failed != null) return failed;
        return ResponseResult.success();
    }

    @Override
    public ResponseResult stop(IdDTO dto) {
        ResponseResult<Object> failed = updateMessageStatus(dto,2);
        if (failed != null) return failed;
        return ResponseResult.success();
    }

    @Override
    public ResponseResult delete(IdDTO dto) {
        removeById(dto.getId());
        return ResponseResult.success();
    }

    private ResponseResult<Object> updateMessageStatus(IdDTO dto,int status) {
        PushMessage pushMessage = getById(dto.getId());
        if (pushMessage==null){
            return ResponseResult.failed("未找到消息推送信息!");
        }
        UpdateWrapper<PushMessage> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda()
                .set(PushMessage::getMessageStatus,status)
                .eq(PushMessage::getId, dto.getId());
        update(updateWrapper);
        return null;
    }
}
